@@ -1,28 +1,32 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import copy
 import datetime
+from decimal import Decimal
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard
-from trytond.pyson import PYSONEncoder
+from trytond.wizard import Wizard, StateView, StateAction, Button
+from trytond.pyson import PYSONEncoder, Eval, Or
 from trytond.transaction import Transaction
-from trytond.tools import safe_eval
+from trytond.tools import safe_eval, reduce_ids
 from trytond.pool import Pool
 
 
 class Template(ModelSQL, ModelView):
     _name = "product.template"
 
-    quantity = fields.Function(fields.Float('Quantity'), 'get_quantity')
+    quantity = fields.Function(fields.Float('Quantity'), 'sum_product')
     forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
-            'get_quantity')
+            'sum_product')
+    cost_value = fields.Function(fields.Numeric('Cost Value'),
+        'sum_product')
 
-    def get_quantity(self, ids, name):
+    def sum_product(self, ids, name):
         res = {}
-        if name not in ('quantity', 'forecast_quantity'):
+        if name not in ('quantity', 'forecast_quantity', 'cost_value'):
             raise Exception('Bad argument')
 
         for template in self.browse(ids):
-            res[template.id] = 0.0
+            res[template.id] = 0. if name != 'cost_value' else Decimal(0)
             for product in template.products:
                 res[template.id] += product[name]
         return res
@@ -33,6 +37,14 @@ class Template(ModelSQL, ModelView):
                 'change_default_uom': 'You cannot change the default uom for '\
                     'a product which is associated to stock moves.',
             })
+        self.cost_price = copy.copy(self.cost_price)
+        self.cost_price.states = copy.copy(self.cost_price.states)
+        self.cost_price.states['required'] = Or(
+            self.cost_price.states.get('required', True),
+            Eval('type').in_(['goods', 'assets']))
+        self.cost_price.depends = copy.copy(self.cost_price.depends)
+        self.cost_price.depends.append('type')
+        self._reset_columns()
 
     def write(self, ids, vals):
         move_obj = Pool().get('stock.move')
@@ -59,6 +71,7 @@ class Template(ModelSQL, ModelView):
 
 Template()
 
+
 class Product(ModelSQL, ModelView):
     _name = "product.product"
 
@@ -66,6 +79,8 @@ class Product(ModelSQL, ModelView):
             searcher='search_quantity')
     forecast_quantity = fields.Function(fields.Float('Forecast Quantity'),
             'get_quantity', searcher='search_quantity')
+    cost_value = fields.Function(fields.Numeric('Cost Value'),
+        'get_cost_value')
 
     def get_quantity(self, ids, name):
         date_obj = Pool().get('ir.date')
@@ -103,7 +118,7 @@ class Product(ModelSQL, ModelView):
         if operator not in ("=", ">=", "<=", ">", "<", "!="):
             return False
         if operator == "=":
-            operator= "=="
+            operator = "=="
         return (safe_eval(str(value) + operator + str(operand)))
 
     def search_quantity(self, name, domain=None):
@@ -131,15 +146,31 @@ class Product(ModelSQL, ModelView):
 
         processed_lines = []
         for (location, product), quantity in pbl:
-            processed_lines.append({'location': location, #XXX useful ?
-                                    'product': product,
-                                    name: quantity})
+            processed_lines.append({
+                    'location': location,  # XXX useful ?
+                    'product': product,
+                    name: quantity,
+                    })
 
-        res= [line['product'] for line in processed_lines \
-                    if self._search_quantity_eval_domain(line, domain)]
+        res = [line['product'] for line in processed_lines
+            if self._search_quantity_eval_domain(line, domain)]
         return [('id', 'in', res)]
 
-
+    def get_cost_value(self, ids, name):
+        cost_values = {}
+        context = {}
+        trans_context = Transaction().context
+        if 'stock_date_end' in context:
+            context['_datetime'] = trans_context['stock_date_end']
+        with Transaction().set_context(context):
+            for product in self.browse(ids):
+                # The date could be before the product creation
+                if not isinstance(product.cost_price, Decimal):
+                    cost_values[product.id] = None
+                else:
+                    cost_values[product.id] = (Decimal(str(product.quantity))
+                        * product.cost_price)
+        return cost_values
 
     def products_by_location(self, location_ids, product_ids=None,
             with_childs=False, skip_zero=True):
@@ -154,11 +185,11 @@ class Product(ModelSQL, ModelView):
                     missing).
                 stock_assign: if set compute also the assigned moves as done.
                 forecast: if set compute the forecast quantity.
-                stock_destinations: A list of location ids. If set, restrict the
-                    computation to moves from and to those locations.
+                stock_destinations: A list of location ids. If set, restrict
+                    the computation to moves from and to those locations.
                 stock_skip_warehouse: if set, quantities on a warehouse are no
-                    more quantities of all child locations but quantities of the
-                    storage zone.
+                    more quantities of all child locations but quantities of
+                    the storage zone.
 
         :param location_ids: the ids of locations
         :param product_ids: the ids of the products
@@ -169,8 +200,7 @@ class Product(ModelSQL, ModelView):
                 and quantity as value
         """
         pool = Pool()
-        uom_obj = pool.get("product.uom")
-        product_obj = pool.get("product.product")
+        uom_obj = pool.get('product.uom')
         rule_obj = pool.get('ir.rule')
         location_obj = pool.get('stock.location')
         date_obj = pool.get('ir.date')
@@ -255,9 +285,9 @@ class Product(ModelSQL, ModelView):
                             '(' \
                                 '(effective_date IS NULL) '\
                             'AND '\
-                                '(COALESCE(planned_date, %s) <= %s) '\
+                                '(planned_date <= %s) '\
                             'AND '\
-                                '(COALESCE(planned_date, %s) >= %s)'\
+                                '(planned_date >= %s)'\
                             ')'\
                         'OR '\
                             '(' \
@@ -273,8 +303,7 @@ class Product(ModelSQL, ModelView):
                 'done', context.get('stock_assign') and 'assigned' or 'done',
                 today, today,
                 'done', 'assigned', 'draft',
-                datetime.date.max, context['stock_date_end'],
-                datetime.date.max, today,
+                context['stock_date_end'], today,
                 context['stock_date_end'], today,
                 ]
 
@@ -356,18 +385,16 @@ class Product(ModelSQL, ModelView):
                     context['stock_date_start'], today,
                     ])
         else:
-            with Transaction().set_user(0, set_context=True):
-                period_ids = period_obj.search([
-                    ('date', '<', context['stock_date_end']),
-                    ('state', '=', 'closed'),
-                ], order=[('date', 'DESC')], limit=1)
-                if period_ids:
-                    period_id, = period_ids
-                    period = period_obj.browse(period_id)
-                    state_date_clause += (' AND '
-                        '(COALESCE(effective_date, planned_date, %s) > %s)')
-                    state_date_vals.extend([datetime.date.max, period.date])
-                    period_vals[0] = period.id
+            period_ids = period_obj.search([
+                ('date', '<', context['stock_date_end']),
+            ], order=[('date', 'DESC')], limit=1)
+            if period_ids:
+                period_id, = period_ids
+                period = period_obj.browse(period_id)
+                state_date_clause += (' AND '
+                    '(COALESCE(effective_date, planned_date) > %s)')
+                state_date_vals.append(period.date)
+                period_vals[0] = period.id
 
         if with_childs:
             query, args = location_obj.search([
@@ -386,9 +413,9 @@ class Product(ModelSQL, ModelView):
         product_template_join = ""
         product_template_join_period = ""
         if product_ids:
-            where_clause += "AND product in (" + \
-                ",".join(('%s',) * len(product_ids)) + ")"
-            where_vals += product_ids
+            red_clause, red_vals = reduce_ids('product', product_ids)
+            where_clause += "AND " + red_clause
+            where_vals += red_vals
         else:
             where_clause += "AND product_template.active = %s"
             where_vals.append(True)
@@ -404,7 +431,6 @@ class Product(ModelSQL, ModelView):
                 "JOIN product_template "
                     "ON (product_product.template = product_template.id) ")
 
-
         if context.get('stock_destinations'):
             destinations = context.get('stock_destinations')
             dest_clause_from = " AND from_location in ("
@@ -477,25 +503,21 @@ class Product(ModelSQL, ModelView):
             leafs = set(all_location_ids)
             parent = {}
             for location in locations:
-                if not location.parent: continue
+                if not location.parent:
+                    continue
                 if location.parent.id in leafs:
                     leafs.remove(location.parent.id)
                 parent[location.id] = location.parent.id
-            locations = set((l.id for l in locations))
+
             while leafs:
+                next_leafs = set()
                 for l in leafs:
-                    locations.remove(l)
                     if l not in parent:
                         continue
+                    next_leafs.add(parent[l])
                     for product in res_product_ids:
                         res.setdefault((parent[l], product), 0)
-                        res[(parent[l], product)] += res.get((l,product), 0)
-                next_leafs = set(locations)
-                for l in locations:
-                    if l not in parent:
-                        continue
-                    if parent[l] in next_leafs and parent[l] in locations:
-                        next_leafs.remove(parent[l])
+                        res[(parent[l], product)] += res.get((l, product), 0)
                 leafs = next_leafs
 
             # clean result
@@ -518,14 +540,12 @@ class Product(ModelSQL, ModelView):
                 all_product_ids = product_ids
             else:
                 all_product_ids = Pool().get("product.product").search([])
-            keys = ((l,p) for l in location_ids for p in all_product_ids)
+            keys = ((l, p) for l in location_ids for p in all_product_ids)
             for location_id, product_id in keys:
                 if (location_id, product_id) not in res:
                     res[(location_id, product_id)] = 0.0
 
         if wh_to_add:
-            if product_ids is None:
-                product_ids = set((p for s, p in res))
             for wh, storage in wh_to_add.iteritems():
                 for product in product_ids:
                     if (storage, product) in res:
@@ -547,9 +567,10 @@ class Product(ModelSQL, ModelView):
 Product()
 
 
-class ChooseStockDateInit(ModelView):
-    _name = 'stock.product_stock_date.init'
-    _description = "Compute stock quantities"
+class ProductByLocationStart(ModelView):
+    'Product by Location'
+    _name = 'product.by_location.start'
+    _description = __doc__
     forecast_date = fields.Date(
         'At Date', help='Allow to compute expected '\
             'stock quantities for this date.\n'\
@@ -560,46 +581,46 @@ class ChooseStockDateInit(ModelView):
         date_obj = Pool().get('ir.date')
         return date_obj.today()
 
-ChooseStockDateInit()
+ProductByLocationStart()
 
-class OpenLocation(Wizard):
-    'Products by Locations'
-    _name = 'stock.location.open'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'stock.product_stock_date.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('open', 'Open', 'tryton-ok', True),
-                ],
-            },
-        },
-        'open': {
-            'result': {
-                'type': 'action',
-                'action': '_action_open_location',
-                'state': 'end',
-            },
-        },
-    }
 
-    def _action_open_location(self, data):
-        model_data_obj = Pool().get('ir.model.data')
-        act_window_obj = Pool().get('ir.action.act_window')
-        act_window_id = model_data_obj.get_id('stock',
-                'act_location_quantity_tree')
-        res = act_window_obj.read(act_window_id)
+class ProductByLocation(Wizard):
+    'Product by Location'
+    _name = 'product.by_location'
+
+    start = StateView('product.by_location.start',
+        'stock.product_by_location_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Open', 'open', 'tryton-ok', default=True),
+            ])
+    open = StateAction('stock.act_location_quantity_tree')
+
+    def do_open(self, session, action):
+        product_obj = Pool().get('product.product')
+        lang_obj = Pool().get('ir.lang')
 
         context = {}
-        context['product'] = data['id']
-        if data['form']['forecast_date']:
-            context['stock_date_end'] = data['form']['forecast_date']
+        product_id = Transaction().context['active_id']
+        context['product'] = product_id
+        if session.start.forecast_date:
+            context['stock_date_end'] = session.start.forecast_date
         else:
             context['stock_date_end'] = datetime.date.max
-        res['pyson_context'] = PYSONEncoder().encode(context)
+        action['pyson_context'] = PYSONEncoder().encode(context)
+        product = product_obj.browse(product_id)
 
-        return res
+        for code in [Transaction().language, 'en_US']:
+            lang_ids = lang_obj.search([
+                    ('code', '=', code),
+                    ])
+            if lang_ids:
+                break
+        lang = lang_obj.browse(lang_ids[0])
+        date = lang_obj.strftime(context['stock_date_end'],
+            lang.code, lang.date)
 
-OpenLocation()
+        action['name'] += ' - %s (%s) @ %s' % (product.rec_name,
+            product.default_uom.rec_name, date)
+        return action, {}
+
+ProductByLocation()
